/*
 * Copyright (c) 1996,1997 Southeastern Universities Research Association,
 *                         Continuous Electron Beam Accelerator Facility
 *
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 *
 *
 *  Revision History:
 *  08/05/00 : Release version 0.4. (Initial version)
 */

package	adm;

import	java.io.*;
import	java.util.*;
import	java.awt.*;

import  cdev.*;

/**
 * CAxis is a class to represent an Axis of a Cartesian widget.  
 * The axis can be either horizontal or vertical. The label for the axis 
 * can be specified to be top or bottom for horizontal axis and left or 
 * right for vertical axis. 
 * <BR><BR>
 * Once created, an Axis must be added (asigned, registered...) to a 
 * Cartesian widget so that the Axis can get paint references and axis 
 * limits.  The limits can be user determined, auto, or determined from 
 * the data. 
 *
 *
 * @version   0.4  08/10/00
 * @author    Ying Chen, Chip Watson
 *             (Based on work by Cuong Quach and Mr. John Winans)
 * @since     adm0.4
 */                                       
class Axis extends Canvas implements CConstants {

    static final int LABEL_GAP = 4;   //gap between tic mark and label  
    static final int TIC_HEIGHT = 5;  //major tic height
    static final Font DEFAULT_FONT = new Font("TimeRoman",Font.PLAIN,12);

    //--- The title of the axis aka the axis label -----------------
    //private String title = "";    //need later
    //private boolean showLabels = true, showMinorTics = false;
    private double displayLo = 0, displayHi = 100;   
    private int minorTick = 5, majorTick = 10; 
    private int axisLength = 100;    //the pixel length to draw this axis
    private int axisStyle = AXIS_STYLE_LINEAR;
    private int position = AXIS_LEFT;
    private int sX, sY, eX, eY;

    //--- for axis label -----------------------------------
    private Font font = null; 
    private int fontH = 0, stringLoW = 0, stringHiW = 0;
    private String stringLo, stringHi;
    private Color labelColor = Color.black;

    /**
     * Null Constructor
     */
    public Axis() { 
	this (AXIS_LEFT);
    }

    /**
     * Constructor with orietation option
     *
     * @param axisPos  The axis position, which may be one of:
     *                 <ol>
     *                    <li>CConstants.AXIX_LEFT
     *                    <li>CConstants.AXIS_RIGHT
     *                    <li>CConstants.AXIX_BOTTOM
     *                    <li>CConstants.AXIS_TOP
     *                 </ol>
     */
    public Axis (int axisPos) {
	position = axisPos;
    }

    /**
     * Create an Axis by specifying some necessary data items.
     *
     * @param min      The axis range low 
     * @param max      The axis range high
     * @param axisPos  The axis position, which may be one of:
     *                 <ol>
     *                    <li>CConstants.AXIX_LEFT
     *                    <li>CConstants.AXIS_RIGHT
     *                    <li>CConstants.AXIX_BOTTOM
     *                    <li>CConstants.AXIS_TOP
     *                 </ol>
     */
    public Axis (double min, double max, int axisDir) {
	this (axisDir);
	displayLo = Math.min(min, max);
	displayHi = Math.max(min, max);
    }

    /**
     * Create an Axis by specifying all necessary data items.
     * @param style    The style of axis,  which should be one of:
     *                 <ol>
     *                    <li>CConstants.AXIS_STYLE_LINEAR
     *                    <li>CConstants.AXIS_STYLE_LOG10
     *                 </ol>
     * @param min      The axis range low 
     * @param max      The axis range high
     * @param axisPos  The axis position, which may be one of:
     *                 <ol>
     *                    <li>CConstants.AXIX_LEFT
     *                    <li>CConstants.AXIS_RIGHT
     *                    <li>CConstants.AXIX_BOTTOM
     *                    <li>CConstants.AXIS_TOP
     *                 </ol>
     */
    public Axis (int style, double min, double max, int axisPos) {
	this (axisPos);
	setAxisStyle(style);
	displayLo = Math.min(min, max);
	displayHi = Math.max(min, max);
    }

    public void setFont(Font f) { 
	font = f;
    }
	
    /**
     * Set the orietation for this axis.
     * @param axisPos   the axis position, which may be one of:
     *                  <ol>
     *                    <li>CConstants.AXIX_LEFT
     *                    <li>CConstants.AXIS_RIGHT
     *                    <li>CConstants.AXIX_BOTTOM
     *                    <li>CConstants.AXIS_TOP
     *                  </ol>
     */
    public void setPosition (int axisPos) {
	position = axisPos;
    }

    public int getPosition () {
	return position;
    }

    /**
     * Set the axis style.
     * @param style   the axis style, which may be one of:
     *                  <ol>
     *                     <li>CConstants.AXIS_STYLE_LINEAR
     *                     <li>CConstants.AXIS_STYLE_LOG10
     *                  </ol>
     */
    public void setAxisStyle (int style) {
	axisStyle = style;
    }

    /**
     * Return the axis style 
     */
    public int getAxisStyle () {
	return axisStyle;
    }

    /**
     * Set the y axis label color to newColor (default is black)
     * @param newColor   the color of y axis label
     */
    public void setLabelColor (Color newColor) {
	labelColor = newColor;
    }

    /**
     * Return the axis label color 
     */
    public Color getLabelColor () {
	return labelColor;
    }


    /**
     * Set the number of major tick marks on axis.  
     * @param cnt   the number of major ticks on the axis.
     */
    public void setMajorTick (int cnt) {
	if (cnt > 0)  majorTick = cnt;
    }
    public int getMajorTick () { return majorTick; }

    /**
     * Set the number of minor tick marks on axis.  
     * @param cnt   the number of minor tiks on the axis.
     */
    public void setMinorTick (int cnt) {
	if (cnt > 0)  minorTick = cnt;
    }
    public int getMinorTick () { return minorTick; }

    /**
     *  Set the display range low and high
     *  @param min   the display range low
     *  @param max   the display range high
     */
    public void setDisplayRange (double min, double max) {
     	displayLo = min;    //Math.min(min, max);
	displayHi = max;    //Math.max(min, max);
	if (font == null) font = DEFAULT_FONT;
	FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
	stringLo = CUtility.format(displayLo);
	stringHi = CUtility.format(displayHi);
	stringLoW = fm.stringWidth(stringLo);
	stringHiW = fm.stringWidth(stringHi);	
	fontH = fm.getHeight();
    }


    /**
     *  Set the display range low 
     *  @param min   the display range low
     */
    public void setDisplayRangeLow (double min) {
     	setDisplayRange (min, displayHi);
	
    }

    /**
     *  Set the display range low 
     *  @param min   the display range low
     */
    public void setDisplayRangeHigh (double max) {
	setDisplayRange (displayLo, max);
    }


    /**
     * Returns the screen width (Pixels in X direction) required to draw 
     * this axis. If the Axis is vertical, the width of the label+tic mark 
     * is returned; if the axis is horizontal, the axisLength is returned.
     */
    public int getWidth () {
	int width;
	if (position == AXIS_BOTTOM || position == AXIS_TOP)
	    width = axisLength;
	else 
	    width = TIC_HEIGHT+LABEL_GAP+Math.max(stringLoW, stringHiW);
	return width;
    }


    /**
     * Returns the screen height (Pixels in X direction) required to draw 
     * this axis. If the Axis is horizontal, the height of the label+tic 
     * mark is returned; if the axis is vertical, the axisLength is returned.
     */
    public int getHeight () {
	int height;
	if (position == AXIS_LEFT || position == AXIS_RIGHT) 
	    height = axisLength;
	else 
	    height = TIC_HEIGHT + LABEL_GAP + fontH;
	return height;
    }
 
    /**
     * set the length of the axis which equal to the width if HORIZONTAL 
     * and the height if VERTICAL
     * @params   len    the length of the axis
     */
    public void setAxisLength (int len) {
	axisLength = len;
    }

    /**
     * Return the length of the axis which equal to the width if HORIZONTAL 
     * and the height if VERTICAL
     */
    public int getAxisLength () { 
	return axisLength; 
    }
   
    public void paint (Graphics g) {
	initGraphics ();
	drawAxisTick (g);
    }
   
    protected void initGraphics () {
	Point l = getLocation();
	switch (position) {
	case AXIS_BOTTOM:
	    sY = l.y;
	    eY = l.y + TIC_HEIGHT;
	    break;
	case AXIS_TOP:
	    int h = getHeight();
	    sY = l.y + h - TIC_HEIGHT;
	    eY = l.y + h;
	    break;
	case AXIS_RIGHT:
	    sX = l.x; 
	    eX = l.x + TIC_HEIGHT;
	    break;
	default:    // case AXIS_LEFT: 
	    int w = getWidth();   
	    sX = l.x + w - TIC_HEIGHT;
	    eX = l.x + w;
	}
	//System.out.println("Axis:: location X="+l.x+", Y="+l.y+
	//		   ", W="+getWidth()+", H="+getHeight());
    }
    
    protected void drawAxisTick (Graphics g) {
	Point l = getLocation();
	int h = getHeight();
	int w = getWidth();  
	g.setColor(Color.black);
	g.setFont(font);
	double inc = ((double)axisLength)/minorTick;	
	switch (position) {
	case AXIS_BOTTOM:
	    g.drawLine(l.x, l.y, l.x+axisLength, l.y);
	    for (int i = 0; i <= minorTick; i++) {
		sX = l.x + (int)(i*inc);
		eX = sX;
		g.drawLine (sX, sY, eX, eY);
	    }	   
	    g.setColor(labelColor);
	    g.drawString(stringLo, l.x, l.y+h);
	    g.drawString(stringHi, l.x+axisLength-stringHiW, l.y+h);
	    break;
	case AXIS_TOP:
	    g.drawLine(l.x, l.y+h, l.x+axisLength, l.y);
	    for (int i = 0; i <= minorTick; i++) {
		sX = l.x + (int)(i*inc);
		eX = sX;
		g.drawLine (sX, sY, eX, eY);
	    }	    
	    g.setColor(labelColor);
	    g.drawString(stringLo, l.x, l.y+fontH);
	    g.drawString(stringHi, l.x+axisLength-stringHiW, l.y+fontH);
	    break;
	case AXIS_RIGHT:
	    g.drawLine(l.x, l.y, l.x, l.y+axisLength);
	    for (int i = 0; i <= minorTick; i++) {
		sY = l.y + (int)(i*inc);
		eY = sY;
		g.drawLine (sX, sY, eX, eY);
	    }
	    g.setColor(labelColor);
	    g.drawString(stringHi, l.x+LABEL_GAP+TIC_HEIGHT, l.y+fontH);
	    g.drawString(stringLo, l.x+LABEL_GAP+TIC_HEIGHT, l.y+axisLength);
	    break;
	default:      // case AXIS_LEFT:
	    g.drawLine(l.x+w, l.y, l.x+w, l.y+axisLength);
	    for (int i = 0; i <= minorTick; i++) {
		sY = l.y + (int)(i*inc);
		eY = sY;
		g.drawLine (sX, sY, eX, eY);
	    }
	    g.setColor(labelColor);
	    g.drawString(stringHi, l.x, l.y+fontH);
	    g.drawString(stringLo, l.x, l.y+axisLength);
	}  
    }  


}  // end of CAxis class

