/**
 * Copyright (c) 2000 Southeastern Universities Research Association,
 *                    Continuous Electron Beam Accelerator Facility
 *
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 *
 *  Revision History:
 *  03/15/00:  Created.
 *  04/02/00:  Ying Chen modified and add comments
 *  08/25/00:  Fix the problem by jtest.
 *
 */

package adm;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import cdev.*;
import cdev.data.*;


/**
 * CButton is an awt-like Button widget. It does not post monitorOn but 
 * sets the device property to a specified value when pressed. 
 * <BR><BR>
 * There is a stringValue and label text associated with the CButton. 
 * When the button is pressed, the device/property in the control system 
 * will be set to the user specified value (stringValue). The StringValue 
 * can be a String or numerical number, such as int, float or double 
 * depending on the nature of the device property. But it must be treated
 * as a String and set through the setStringValue (String val) method. 
 * CButton is responsible to convert this StringValue to a correct data type 
 * prior to the set action. If the set command fails due to the convertion 
 * or a permission problem, the button will be disabled and the label color 
 * of the button will be set to red. 
 * 
 *
 * @version   0.4  08/29/00
 * @author    Ying Chen, Cuong Quach, Chip Watson,
 * @since     adm0.2
 */
public class CButton extends Button implements CInterface,
					       DeviceListener,
					       ActionListener {
    private static int VAL_TAG = Data.convertKey("value");
    private static String BLANK = "";

    private Device device = null;
    private String deviceName = null;
    private String propertyName = null;
    private String stringValue = null;
    private Color  normalColor = this.getForeground();
    private Color  activatedColor = Color.red;
    private Color  deadControlColor = Color.white;
    private boolean firstCall = true;  
    private int dataType;

    /**
     * Constructor
     */
    public CButton()
    {
	super();
	setVisible(true);
	addActionListener(this);
    }


    /**
     * Null constructor  with device name and property name
     *
     * @param dName    device name
     * @param pName    property name
     */
    public CButton(String dName, String pName)  {
	this();
	setDeviceName(dName);
	setPropertyName(pName);
    }


    /**
     * Set the value that gets sent to the control system when 
     * the button is pressed.
     * @param v    the string/value to send
     */
    public void setStringValue (String v)
    {
	stringValue = v;  
    }

    /**
     * Return the the string/value associated with the button
     */
    public String getStringValue () { return stringValue; }


    protected void getFirstValue () 
    {         
	if (deviceName == null || propertyName == null)  return;
	try { 
	    device = new Device(deviceName);
	    device.get(propertyName, this);
	}
	catch (IOException e) { 
	    System.out.println("Invalid device.property ("+
			       deviceName+"."+propertyName+")");
	    setForeground(deadControlColor);
	    setEnabled(false);
	}
    }


    /**
     * Set the Device name 
     * @param newName     the name of device you want to connect
     */
    synchronized public void setDeviceName(String newName) 
    {
       newName = CUtility.makeValidName(newName);
       if (!CUtility.testNameMatch(deviceName, newName)) {
          deviceName = newName;
	  getFirstValue();
       }
    }

    /**
     * Return the device name to which this component is connected.
     * If device name is not set, return empty string "". 
     */
    public String getDeviceName()
    {
	return (deviceName==null)?BLANK:deviceName;
    }
    

    /**
     * Set device property name 
     * @param newName    the name of device property to monitor
     */  
    synchronized public void setPropertyName (String newName)
    {
	newName = CUtility.makeValidName(newName);
        if (!CUtility.testNameMatch(propertyName, newName)) {
            propertyName = newName;
	    getFirstValue ();
	}
    }
    

    /**
     * Return the device property name to which this component is connected.
     * If property name is not set, return empty string "". 
     */
    public String getPropertyName()
    {
	return (propertyName==null)?BLANK:propertyName;
    }
    

    /**
     *  Send a value to the control system.
     */
    public void actionPerformed (ActionEvent evt)
    {
	Data tdata = new cdev.data.Data();
	try {
	    if (dataType == Data.DOUBLE) {
		double value = Double.valueOf(stringValue).doubleValue();
		tdata.insert(VAL_TAG, value);
		setValue(tdata);
	    }
	    else if (dataType == Data.FLOAT) {
	    	float value = Float.valueOf(stringValue).floatValue();
		tdata.insert(VAL_TAG, value);
		setValue(tdata);
	    }
	    else if (dataType == Data.INT) {
		int value = Integer.parseInt(stringValue);
		tdata.insert(VAL_TAG, value);
		setValue(tdata);
	    }
	    else if (dataType == Data.LONG) {
		long value = Long.parseLong(stringValue);
		tdata.insert(VAL_TAG, value);
		setValue(tdata);
	    }
	    else if (dataType == Data.STRING) {
		tdata.insert(VAL_TAG, stringValue);
		setValue(tdata);
	    }
	    else {
		System.out.println("The data type of "+deviceName+"."
				   +propertyName+
				   " is not supported in this version");
		setForeground(deadControlColor);
		setEnabled(false);
	    }
	}
	catch (NumberFormatException ne) {
	    System.out.println("The data type of \""+stringValue
			       +"\" is not compatible with "
			       +deviceName+"."+propertyName
			       +" in the control system.");
	    setForeground(deadControlColor);
	    setEnabled(false);
	}  
    }
	
    private void setValue (Data d) 
    {
	try {
	    device.set(propertyName, d, (DeviceListener)this);
	    device.getControlSystem().flush();
	    setForeground(activatedColor);
	} catch (Exception e) {
	    System.out.println("CButton::setValue: failed to set value: "+e);
	    setForeground(deadControlColor);
	}
	setEnabled(false);
    }


    /**
     * Check the error message back from device.set(..) 
     */
    public void deviceChanged(DeviceEvent de) {
	if (firstCall) {
	    try {
		dataType=de.getData().getDataEntry("value").getEnumeratedType();
		firstCall = false;
	    }
	    catch (Exception e) { 
		 setForeground(deadControlColor);
		 setEnabled(false);
	    }
	}
	else {
	    if (de.getError() == null) {
		System.out.println("Success to send value ("+stringValue+").");
		setForeground(normalColor);
		setEnabled(true);
	    } else {
		System.out.println("Failed to send value ("+stringValue+").");
		setForeground(deadControlColor);
	    }
	}
    }    

}  // end CButton



