/*
 * Copyright (c) 2000 Southeastern Universities Research Association,
 *                    Continuous Electron Beam Accelerator Facility
 *
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 *
 *
 *  Revision History:
 *    03/10/00: Release version 0.1.
 *    04/17/00: Release version 0.2. 
 *    05/21/00: Release version 0.3 
 *              (move some codes to CAnimatedCanvas class)
 */

package	adm;

import	java.io.*;   
import	java.awt.*;

import  cdev.*;
import  cdev.data.*;


/**
 * CCanvas is a abstract class. It is responsible for posting the 
 * minotorOn and receiving data. It is the base class of the control 
 * system display widgets. It uses awt.component paint and update methods.
 * <BR><BR> 
 * Users can extend this class to develop simple custom widgets. 
 * The derived classes must implement the deviceChanged method.
 * 
 *
 * @version   0.2  04/15/00
 * @author    Ying Chen, Chip Watson
 *            (Based on work by Cuong Quach and Mr. John Winans)
 * @since     adm0.1
 */
public abstract class CCanvas extends Canvas
	implements CInterface, DeviceListener, CConstants {

    protected static DeviceContext defContext = new DeviceContext();
    protected static DeviceContext limitsContext;
    static {
      Data limitsContextData = new Data();
      limitsContextData.insert("displayHigh", 1);
      limitsContextData.insert("displayLow", 1);
      limitsContextData.insert("controlHigh", 1);
      limitsContextData.insert("controlLow", 1);
      limitsContextData.insert("alarmHigh", 1);
      limitsContextData.insert("alarmLow", 1);
      limitsContextData.insert("warningHigh", 1);
      limitsContextData.insert("warningLow", 1);
      limitsContextData.insert("precision", 1);
      limitsContext = new DeviceContext(limitsContextData);
    }
    static final int MINSIZE = 5;

    static final Color DEFAULT_WARNING_COLOR = Color.yellow;
    static final Color DEFAULT_ALARM_COLOR = Color.red;
 
    //--- for CInterface ------------------------------
    protected static String BLANK = "";
    protected boolean active = false;
    protected cdev.Device device = null;
    protected String deviceName = null;
    protected String propertyName = null;
    protected String lastDeviceName = null;
    protected String lastPropertyName = null;

    //--- define default value --------------------------
    protected Color warningColor = DEFAULT_WARNING_COLOR;
    protected Color alarmColor   = DEFAULT_ALARM_COLOR;
    protected int colorMode  = CLRMOD_STATIC;
    protected int labelStyle = LABEL_NONE;
    protected int rangeStyle = RANGE_FROM_PROPERTY;
    protected int frameW = 2;           // dafault face frame = 2 pixels
    protected double controlHi = 100.0;
    protected double controlLo = 0.0;
    protected double displayHi = 90.0;
    protected double displayLo = 10.0;
    protected double alarmHi = 80.0;
    protected double alarmLo = 20.0;
    protected double warningHi = 70.0;
    protected double warningLo = 30.0;
    protected double rangeHi = displayHi;
    protected double rangeLo = displayLo;
    protected double precision = 1.0;
    protected Color fgColor = Color.green;
    protected Color bgColor = Color.gray;
    protected Font font =  new Font("Dialog", Font.PLAIN, 12);
    
    //--- boolean flag -------------------------------------------
    protected   boolean firstTime = true;   
    protected   boolean firstCall = true; 
    protected   boolean gotLimits = false;

    protected double currVal = 0;
    protected Color currColor = null;


    /**
     * Null Constructor
     */
    protected CCanvas() 
    {
	this(0, 0);
    }
  
    /**
     * Constructor with device name and property name
     * 
     * @param dName    device name
     * @param pName    property name
     */
    public CCanvas(String dName, String pName) 
    {
	setSize (0, 0);
	setDeviceAndPropertyName(dName, pName);
    }
    
    /**
     * Constructor
     * Defines the canvas size and uses default foreground and 
     * background colors.
     * @param width    component width
     * @param height   component height
     */
    public CCanvas(int width, int height) 
    {
	setSize (width, height);
    }
    

    /**
     * Constructor
     * Defines the canvas size and sets foreground and background colors.
     * @param width     component width
     * @param height    component height
     * @param fg        foreground color
     * @param bg        background color
     */
    public CCanvas( int width, int height, Color fg, Color bg )
    {
	this(width, height);
        if (fg != null) fgColor = fg;
        if (bg != null) bgColor = bg;
    }


    /**
     * Set limitsContext to get all meta-data at first time
     */
    synchronized protected void getFirstValue () 
    {
       try {
	  if (deviceName == null || propertyName == null)   return;
          if (!deviceName.equals(lastDeviceName)) {
             device = new Device(deviceName);
             lastDeviceName = deviceName;
             }
          device.setContext (limitsContext);
          device.get(propertyName, this);
          firstCall = true; 
          }
       catch(Exception e) { e.printStackTrace(); }
     }
     

    /**
     * Set the Device name 
     * @param newName     the name of device you want to connect
     */
    synchronized public void setDeviceName(String newName) 
    {
       newName = CUtility.makeValidName(newName);
       if (!CUtility.testNameMatch(deviceName, newName)) {
          stopMonitor();
          deviceName = newName;
          getFirstValue ();
       } 
    }
    

    /**
     * Return the device name to which this component is connected.
     * If device name is not set, return empty string "". 
     */
    public String getDeviceName()
    {
       return (deviceName==null)?BLANK:deviceName;
    }


    /**
     * Set device property name 
     * @param newName    the name of device property you want to connect
     */  
    synchronized public void setPropertyName(String newName)
    {
	newName = CUtility.makeValidName(newName);
        if (!CUtility.testNameMatch(propertyName, newName)) {
           stopMonitor();
           propertyName = newName; 
           getFirstValue ();        
        }
    }


    /**
     * Return the device property name to which this component is connected.
     * If property name is not set, return empty string "". 
     */
    public String getPropertyName()
    {
	return (propertyName==null)?BLANK:propertyName;
    }
	

   /**
     * Set device name and device property name
     * @param newDeviceName    the device name you want to connect
     * @param newPropertyName  the device property name you want to connect
     */  
    synchronized public void setDeviceAndPropertyName
                                       (String newDeviceName,
					String newPropertyName)
    {
	newDeviceName = CUtility.makeValidName(newDeviceName);
	newPropertyName = CUtility.makeValidName(newPropertyName);
        if ((!CUtility.testNameMatch(deviceName, newDeviceName)) ||
           (!CUtility.testNameMatch(propertyName, newPropertyName))) {
           stopMonitor();
	   deviceName = newDeviceName;
           propertyName = newPropertyName; 
           getFirstValue ();        
        }
    }


    protected void stopMonitor() 
    {
        if (active) {
          try {
	     active = false;
	     device.monitorOff(propertyName,this);
	  } catch (IOException e) { e.printStackTrace(); }
	}
    }
	
	
   protected void startMonitor() 
   {
      try {
         if (active) stopMonitor();
         if (deviceName == null || propertyName == null) return;
         if (!deviceName.equals(lastDeviceName)) {
             device = new Device(deviceName);
             lastDeviceName = deviceName;
             }    
         device.setContext (defContext);
         device.monitorOn(propertyName,this);
         lastPropertyName = propertyName;
         active = true;
      }  catch (IOException e) {
	  active = false;
	  e.printStackTrace();
      }
   }
	
}  // end of CCanvas class






