/*
 * Copyright (c) 2000 Southeastern Universities Research Association,
 *                    Continuous Electron Beam Accelerator Facility
 *
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 *
 *  Revision History:
 *  03/10/00:  Release version 0.1.
 *  04/17/00:  Release version 0.2 (add format property, display text & value)
 *  08/16/00:  Release version 0.4 (modify to work with string value)
 *
 */

package adm;

import java.awt.*;
import java.io.*;

import cdev.*;
import cdev.data.*;


/**
 * CLabel is an awt component bean. It is used to display a value of the 
 * specified device property in the control system as a text label. The 
 * size, bound, foreground, background, the font and alignment of the label 
 * can be set by the user. It also can display the specific string if 
 * setText(String) method is called. 
 * <BR><BR>
 * The color of the text will be the same as the foreground color if the 
 * color mode is set to CLRMOD_STATIC (default). The color of the text 
 * will change to yellow or red if the property is in a warning or alarm 
 * state (as determined by warning and alarm limits) when the color mode 
 * is set to CLRMOD_ALARM. 
 * <BR><BR>
 * The display value can be formatted following printf conventions, or 
 * using one of following static format constants: 
 * <OL>
 *     <LI>FRMT_INTEGER (%d) 
 *     <LI>FRMT_FLOAT (%.3f) 
 *     <LI>FRMT_SCIENTIFIC (%.3e) 
 *     <LI>FRMT_AUTO (%.3g) 
 *     <LI>FRMT_HEX (%#x) 
 *     <LI> FRMT_OCTAL (%#o) 
 * </OL>
 *
 * The default format is FRMT_AUTO which means the value will display
 * as a floating point number or scientific notation depending on the 
 * value. 
 *
 *
 * @version   0.2  03/29/00
 * @author    Chip Watson, Ying Chen 
 *            (Based on work by Cuong Quach and Mr. John Winans)
 * @since     adm0.1
 *
 */
public class CLabel extends Label implements 
                   CConstants, CInterface, DeviceListener {
 
    private static String BLANK = "";  
    private static double BAD_VALUE = -11111;
    private static DeviceContext defContext = new DeviceContext();
    private static DeviceContext limitsContext;
    static {
      Data limitsContextData = new Data();
      limitsContextData.insert("displayHigh", 1);
      limitsContextData.insert("displayLow", 1);
      limitsContextData.insert("alarmHigh", 1);
      limitsContextData.insert("alarmLow", 1);
      limitsContextData.insert("warningHigh", 1);
      limitsContextData.insert("warningLow", 1);
      limitsContext = new DeviceContext(limitsContextData);
    }
    static final Color DEFAULT_WARNING_COLOR = Color.yellow;
    static final Color DEFAULT_ALARM_COLOR = Color.red;
    static final Color DEFAULT_FOREGROUND = Color.black;

    private boolean active = false;
    private boolean showValue = true;
    private boolean firstCall = true; 
    private boolean gotLimits = false;

    private cdev.Device device = null;
    private String deviceName = null;
    private String propertyName = null;
    private String lastDeviceName = null;
    private String lastPropertyName = null;
    
    //--- define default value --------------------------
    private Color warningColor = DEFAULT_WARNING_COLOR;
    private Color alarmColor   = DEFAULT_ALARM_COLOR;
    private Color fgColor = DEFAULT_FOREGROUND;
    private int colorMode  = CLRMOD_STATIC;
    private String format = FRMT_AUTO;
    private double displayHi = BAD_VALUE;
    private double displayLo = BAD_VALUE;
    private double alarmHi = BAD_VALUE;
    private double alarmLo = BAD_VALUE;
    private double warningHi = BAD_VALUE;
    private double warningLo = BAD_VALUE;
    private double value = BAD_VALUE;
    private String valueString = null;
    private String textString = null;

    /**
     * Null constructor. If setText is not called, this label will displays 
     * the value of the specified device property.
     */
    public CLabel() 
    {
        setForeground(fgColor);
        super.setText("           ");
    }
    
    
    /**
     * Null constructor  with device name and property name
     *
     * @param dName    device name
     * @param pName    property name
     */
    public CLabel(String dName, String pName)  {
        this();
        setDeviceName(dName);
        setPropertyName(pName);
    }


    /**
     * Constructs a new label with the specified string of text, left justfied
     * @param sText      the text string that this label presents. 
     */
    public CLabel(String aText) 
    {
        setText(aText);
    }

    /**
     * Set the color mode property
     * @param newColorMode   the new color mode, which may be one of:
     *             <OL>
     *             <LI>  CConstants.CLRMOD_STATIC
     *             <LI>  CConstants.CLRMOD_ALARM
     *             </OL>
     */
    final public void setColorMode (int newColorMode) {
        colorMode = newColorMode;
    }
    
    
    /**
     * Return the color mode property
     */
    final public int getColorMode () { return colorMode; }
    
    
    /**
     * Set the text for this label to the specified text.
     * If the sText is not null or "", the label will not show the
     * value retrieved from the control system. Otherwise the 
     * label will be clear (set to "") and device/property value
     * will be displayed.
     *
     * @param sText      the text string that this label presents. 
     */
    public void setText(String aText) 
    {
        if (aText == null || aText.trim().equals(BLANK)) {
            showValue = true;
            super.setText(BLANK);
        }
        else {
            showValue = false;
            textString = aText;
            super.setText(textString);
        }
    }

    /**
     * Return the text format (format is following the printf conventions)
     */
    final public String getFormat() { return format; }

    /**
     * Set the text format. The label will be forced to display the value of 
     * the specified device property.
     * @param format    the format of the label text, which following printf 
     *                  conventions. Or use one of following string:
     *                  <OL>
     *                  <LI> CConstants.FRMT_INTEGER
     *                  <LI> CConstants.FRMT_FLOAT   
     *                  <LI> CConstants.FRMT_SCIENTIFIC
     *                  <LI> CConstants.FRMT_AUTO
     *                  <LI> CConstants.FRMT_HEX   
     *                  <LI> CConstants.FRMT_OCTAL
     *                  </OL>
     */
    final public void setFormat(String aFormat) 
    {
        showValue = true;
        format = aFormat;
    }


    /**
     * Set limitsContext to get all meta-data at first time
     */
    synchronized protected void getFirstValue () 
    {
       try {
          if (deviceName == null || propertyName == null)   return;
          if (!deviceName.equals(lastDeviceName)) {
             device = new Device(deviceName);
             lastDeviceName = deviceName;
             }
          device.setContext (limitsContext);
          device.get(propertyName, this);
          firstCall = true; 
          }
       catch(Exception e) { e.printStackTrace(); }
     }
     

    /**
     * Set the Device name 
     * @param newName     the name of device you want to connect
     */
    synchronized public void setDeviceName(String newName)
    {
        newName = CUtility.makeValidName(newName);
        if (!CUtility.testNameMatch(deviceName, newName)) {
            stopMonitor();
            deviceName = newName;
            getFirstValue ();

        }
    }
    
    /**
     * Return the device name to which this component is connected.
     * If device name is not set, return empty string "". 
     */
    public String getDeviceName()
    {
        return (deviceName==null)?BLANK:deviceName;
    }
    
    /**
     * Set device property name 
     * @param newName    the name of device property you want to connect
     */  
    synchronized public void setPropertyName(String newName)
    {
        newName = CUtility.makeValidName(newName);
        if (!CUtility.testNameMatch(propertyName, newName)) {
            stopMonitor();
            propertyName = newName;
            getFirstValue ();

        }
    }
    
    /**
     * Return the device property name to which this component is connected.
     * If property name is not set, return empty string "". 
     */
    public String getPropertyName()
    {
        return (propertyName==null)?BLANK:propertyName;
    }
  
    
    protected void stopMonitor() {
        if (active) try {
            active = false;
            device.monitorOff(propertyName,this);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    protected void startMonitor() {
        try {
            if (active) stopMonitor();
            if (deviceName == null || propertyName == null) return;
            if (!deviceName.equals(lastDeviceName)) {
                device = new Device(deviceName);
                lastDeviceName = deviceName;
            }
            device.setContext (defContext);
            device.monitorOn(propertyName,this);
            lastPropertyName = propertyName;
            active = true;
        } catch (IOException e) {
            active = false;
            e.printStackTrace();
        }
    }
    
    /**
     * At first call, try to get the limits. 
     * If failed, always use foreground color. 
     * If success, the label color will change according to the value
     * when the color mode is set to CLRMOD_ALARM.
     */
    public void deviceChanged(DeviceEvent evt) {
        if (firstCall) {
            fgColor = getForeground();
            try {
                displayHi=evt.getData().getDataEntry("displayHigh").doubleValue();
                displayLo=evt.getData().getDataEntry("displayLow").doubleValue();
                gotLimits = true;
            }
            catch (Exception e) {
                gotLimits = false;
                System.out.println("Failed to get display range");
            }

            //--- get the alarm and warning range ----------------------
            try {
                alarmHi=evt.getData().getDataEntry("alarmHigh").doubleValue();
                alarmLo=evt.getData().getDataEntry("alarmLow").doubleValue();
            }
            catch  (Exception e) {
                gotLimits = false;
                System.out.println("Failed to get alarm range");
            }
            try {
                warningHi=evt.getData().getDataEntry("warningHigh").doubleValue();
                warningLo=evt.getData().getDataEntry("warningLow").doubleValue();
            }
            catch  (Exception e) {
                gotLimits = false;
                System.out.println("Failed to get warining range");
            }
            firstCall = false;  
            startMonitor();
        }
        else {    // not first call
            setForeground(fgColor);
            try {
                valueString = evt.getData().getDataEntry("value").stringValue();
            } catch (Exception e) { 
                super.setText(BLANK);   
                System.out.println("CLabel:: failed to get the value");
                return;
            }
            try {
                value = Double.valueOf(valueString).doubleValue();
                if ((colorMode == CLRMOD_ALARM) && gotLimits) {
                    if (value-alarmHi > 0 || value-alarmLo < 0) 
			setForeground(alarmColor);
		    else if (value-warningHi > 0 || value-warningLo < 0) 
			setForeground(warningColor);
		}
		if (showValue) 
		    super.setText(CUtility.format(value, format));
		else 
		    super.setText(textString);
	    } catch (Exception e) { 
		if (showValue) 
		    super.setText(valueString);
		else 
		    super.setText(textString);
	    }

	}
    }

    
    /*   
    static public void main(String args[])
    {
	class DriverFrame extends Frame {
	    public DriverFrame() {
		addWindowListener(new java.awt.event.WindowAdapter() {
		    public void windowClosing(java.awt.event.WindowEvent event)
			{
			    dispose();	  // free the system resources
			    System.exit(0); // close the application
			}
		});
		this.setLayout(null);
		this.setSize(400,300);
		this.add(new CLabel());
	    }
	}
	
	new DriverFrame().show();
    }    
    */
}










