/*
 * Copyright (c) 2000 Southeastern Universities Research Association,
 *                    Continuous Electron Beam Accelerator Facility
 *
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 *
 *  Revision History:
 *  03/10/00:  Release version 0.1.
 *  04/20/00:  Release version 0.2.  
 *
 */

package adm;

import java.awt.*;
import cdev.*;


/**
 * CMeter is an awt component bean. It displays a property's value in the 
 * control system as a semi-circular meter object enclosed in a rectangle. 
 * The size, bound, foreground, background and the font of the label on the 
 * widget can be set like an awt component. 
 * <BR><BR>
 * The color of the meter's needle is same as the foreground if the color 
 * mode property is set to CLRMOD_STATIC (default). It will change from 
 * foreground color to yellow or red depending on the value, warning and 
 * alarm limits when CLRMOD_ALARM is used. 
 * <BR><BR>
 * There are four label styles which can be selected, with a default of 
 * LABEL_NONE. If LABEL_AXIS is used, the tick marks and display range 
 * will appear on the widget. In addition, the property's value will also 
 * shown on the widget if LABEL_VALUE is selected. The device and property 
 * name will be added when LABEL_ALL is used. 
 * 
 *
 * @version   0.2  04/17/00
 * @author    Ying Chen, Chip Watson
 *            (Based on work by Cuong Quach and Mr. John Winans)
 * @since     adm0.1
 *
 */
public class CMeter extends CAnimatedCanvas {	
	
    //--- GUI graphics info. --------------------
    private int faceX, faceY, faceW, faceH = 0;  // meter face area
    private int centerX, centerY, radius = 0;	 // meter face data
    private int axisY, axisH = 0;		 // axis label area
    private int valX, valY, valW, valH = 0;	 // value display area
    private int sX, sY, sW, sH = 0;
    private int chaX, chaY, chaW, chaH = 0;	 // channel name area
    private double maxVal, minVal = 0;
    
    private FontMetrics fm;
    private int fontH, fontD = 0;
    private String stringHi, stringLo;
    private String fmt = "%.3g";
	 
    //---- Needle data. ----------------------------
    private Polygon needle = null;
    private int[] xneedle = new int[4];
    private int[] yneedle = new int[4];
    private int   spread = 12;  		// fattest point of needle
    private int   edgeRad, needleRad = 0;
    private int	  dialRad = 0;			// radius of center ball
    private int	  dialDia = 0;			// should be 2*dialRad
    private double  NeedlePos = 0;
    private double[] pretty;

    private boolean firstPoint = true;
    private boolean change = false;
    
    /**
     * Null Constructor.
     */
    public CMeter() 
    {
	super();	
    }

    /**
     * Constructor with device name and property name
     * 
     * @param dName    device name
     * @param pName    property name
     */
    public CMeter(String dName, String pName) 
    {
	super(dName, pName);	
    }

    /**
     * Constructor
     * Defines the canvas size and use default foreground and background color
     * @param width    component width
     * @param height   component height
     */
    public CMeter(int w, int h) 
    {
	super(w, h);	
    }
    
    /**
     * Constructor
     * Defines the canvas size and sets foreground and background colors.
     * @param width     component width
     * @param height    component height
     * @param fg        foreground color
     * @param bg        background color
     */
    public CMeter(int w, int h, Color fg, Color bg) 
    {
	super(w, h, fg, bg);	
    }
    

    private void initGraphics () 
    {	
	Dimension rec = getSize ();
	chaX = 0; chaY = 0; chaW = 0; chaH = 0;
	valX = 0; valY = 0; valW = 0; valH = 0;
	axisY= 0; axisH= 0;

	//--- calculate the first rangeHi/rangeLo when RANGE_AUTO ----
	if ((firstPoint) && (rangeStyle == RANGE_AUTO)) { 
	    maxVal = currVal;   minVal = currVal;  
	    rangeHi = maxVal + Math.abs(0.1*maxVal);
	    rangeLo = minVal - Math.abs(0.1*minVal);
	    firstPoint = false;
	}

	//--- get pretty axis value --------------------------------
	pretty = CUtility.makePrettyAxis(rangeLo, rangeHi);
	rangeLo = pretty[0];
	rangeHi = pretty[1];

	if (labelStyle != LABEL_NONE) {
	    //fm = getFontMetrics(font); // for java 1.2
	    fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
	    fontH = fm.getHeight();
	    fontD = fm.getDescent();
	    stringLo = CUtility.format(rangeLo, fmt);
	    stringHi = CUtility.format(rangeHi, fmt);
	} 
	switch(labelStyle) {
	case LABEL_ALL:
	    chaX = 0;
	    chaY = 0;
	    chaW = rec.width;
	    chaH = fontH;  
	case LABEL_VALUE:
	    valW = rec.width; 
	    valH = fontH - fontD; 
	    valX = 0;
	    valY = rec.height - valH;
	case LABEL_AXIS:
	    //axisY is depend on faceH
	    axisH = fontH; 
	default:        //case LABEL_NONE:
	    faceH = rec.height - chaH - axisH - valH;
	    faceH = Math.min((int)(rec.width/2), faceH);
	    faceW = 2*faceH;
	    faceX = (rec.width - faceW)/2;
	    faceY = chaH;
	    axisY = chaH + faceH;	
	}

	//--- Set up coordinates for needle and dial based on faceX,Y ----
	centerX =  faceX + (faceW/2);
	centerY =  faceY + frameW + (int)(0.95*(faceH - 2*frameW));  
	
	radius = faceH - 2*frameW;
	edgeRad = (int)(radius * 0.2);
	needleRad = (int)(radius * 0.72);
	dialRad = (int)(radius * 0.07);   // radius of spindle about which needle spins
	dialDia = (int)(2 * dialRad);
	
	for (int i=0; i<xneedle.length; i++) {
	    xneedle[i] = centerX;
	    yneedle[i] = centerY;
	}
	needle = new Polygon(xneedle,yneedle,4);
	xneedle = needle.xpoints;
	yneedle = needle.ypoints;	
    }

    
    protected void paintBackground (Graphics g) 
    {
	initGraphics();
	switch(labelStyle) {
	case LABEL_ALL:
	    drawName(g);
	case LABEL_VALUE:
	    drawValue(g);
	case LABEL_AXIS:
	    drawAxisLimit(g);
	default:                //case LABEL_NONE:
	    drawFace(g);
	}
    }
    
    
    protected void paintForeground (Graphics g) 
    {
	if ((labelStyle == LABEL_VALUE)||(labelStyle == LABEL_ALL)) {
	    drawValue(g);
	}
	
	//--- Erase old needle -----------------------
	g.setColor(bgColor.darker());
	g.fillPolygon (needle);

	if (rangeStyle == RANGE_AUTO)   do_auto();

	NeedlePos = (currVal - rangeLo) / (rangeHi - rangeLo);
	NeedlePos = Math.max(NeedlePos, 0.0);  
	NeedlePos = Math.min(NeedlePos, 1.0);   
	int angle = 180 + (int)(NeedlePos * 180);
	
	//--- update coordinates for needle ---------------------
	xneedle[1]=centerX+(int)(edgeRad*Math.cos((angle+spread)*Math.PI/180));
	yneedle[1]=centerY+(int)(edgeRad*Math.sin((angle+spread)*Math.PI/180));
	xneedle[2]=centerX+(int)(needleRad*Math.cos((angle)*Math.PI/180));
	yneedle[2]=centerY+(int)(needleRad*Math.sin((angle)*Math.PI/180));
	xneedle[3]=centerX+(int)(edgeRad*Math.cos((angle-spread)*Math.PI/180));
	yneedle[3]=centerY+(int)(edgeRad*Math.sin((angle-spread)*Math.PI/180));
	
	//--- draw the new needle and fill in the center oval---------
	g.setColor(currColor);
	g.fillPolygon(needle);
	g.fillOval(centerX-dialRad, centerY-dialRad, dialDia, dialDia);
    }
    
    protected void do_auto ()
    {
	double d1 = currVal - rangeHi;
	double d2 = currVal - rangeLo;
        if (d1 > 0) { change = true; maxVal = currVal; }
        if (d2 < 0) { change = true; minVal = currVal; }
        if (change) {
	    pretty = CUtility.makePrettyAxis(minVal, maxVal);
            rangeLo = pretty[0];
            rangeHi = pretty[1];
	    change = false;
	    offGraphics = null;   
            repaint();
        }
    }

    /*
     * Draws meter frame and the marks on the meters face. 
     * The meter is divided into 10 increments in a semi-circle.
     */
    private void drawFace (Graphics g)
    {
	g.setColor(bgColor);
	g.fill3DRect(faceX, faceY, faceW, faceH, true);
	faceX += frameW;
	faceY += frameW;
	faceW -= 2*frameW;
	faceH -= 2*frameW;
	g.setColor(bgColor);
	g.fill3DRect(faceX, faceY, faceW, faceH, false);
	g.setColor(fgColor);
	
	int ix, iy, ox, oy = 0;
	int orad = (int) (radius * 0.85);
	int irad = (int) (radius * 0.75);
	for (int i=0; i<= pretty[2]; i++) {
	    int angle = 180 + (int)(i*(float)180/pretty[2]);
	    double cos = Math.cos(angle * Math.PI / 180);
	    double sin = Math.sin(angle * Math.PI / 180);
	    ix = centerX + (int)( irad * cos );
	    iy = centerY + (int)( irad * sin );
	    ox = centerX + (int)( orad * cos );
	    oy = centerY + (int)( orad * sin );
	    g.drawLine (ix, iy, ox, oy);           // draw the tick marks
	}
    }
    
    
    private void drawAxisLimit(Graphics g) 
    {	
	g.setColor(fgColor);
	g.setFont(font);
	
	//--- draw High and Low label of axis ------------
	int strLoX = faceX + frameW;
	int stringW = fm.stringWidth(stringHi);
	int strHiX = faceX + faceW - stringW - frameW;
	int allY = axisY + axisH;
	g.drawString(stringLo, strLoX, allY );
	g.drawString(stringHi, strHiX, allY );
	
    }
    

    private void drawValue(Graphics g) 
    {
	//--- Erase old fill rect.-------------
	g.setColor (bgColor);
	g.fillRect(sX, valY-fontD, sW, sH);

	String stringCurrVal = CUtility.format(currVal, fmt);
        sW= fm.stringWidth(stringCurrVal);
	sX = valX + (int)((valW - sW)/2);
	sY = valY + valH - fontD;  
	sH = fontH + fontD;
	g.setColor (fgColor);
	g.drawString (stringCurrVal, sX, sY);
    }
    

    private void drawName(Graphics g) 
    {
	String chaName = deviceName+"."+propertyName;
	String tempName = chaName;
	//--- remove the last letter to fit the limited space ----
	int chanLen = fm.stringWidth(tempName);
	int i = 2;
	while (chaW <= chanLen) {
	    tempName = tempName.substring(0,tempName.length()-i);
	    i++;
	    chanLen = fm.stringWidth(tempName);
	}
	
	int stringW = fm.stringWidth(tempName);
	int stringX = chaX + (int)((chaW - stringW)/2);
	int stringY = chaY + chaH - fontD;   
	g.setColor (fgColor);
	g.drawString (tempName, stringX, stringY);		
    }
    
} //end of CMeter








