/*
 * Copyright (c) 1996,1997 Southeastern Universities Research Association,
 *                         Continuous Electron Beam Accelerator Facility
 *
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 *
 *  Revision History:
 *    04/17/00:  Release version 0.2. (Initial version)
 *    08/29/00:  Release version 0.4 (change field to private).
 *
 */

package	adm;

import java.awt.*;
import java.io.*;
import cdev.*;
import cdev.data.*;

/**
 * Pen data model for CStripChart. It does not remember the history 
 * data. It only returns the current data and the met-data. 
 *
 *
 * @version   0.2  03/29/00
 * @author    Chip Watson, Ying Chen 
 *            (Based on work by Cuong Quach and Mr. John Winans)
 * @since     adm0.2
 *
 */                                       
public class CPen extends Component implements CInterface, DeviceListener {

    static final String BLANK = "";
    static final double BAD_VALUE = -11111.0;
    private static DeviceContext defContext = new DeviceContext();
    private static DeviceContext limitsContext;
    static {
        Data limitsContextData = new Data();
        limitsContextData.insert("displayHigh", 1);
	limitsContextData.insert("displayLow", 1);
        limitsContext = new DeviceContext(limitsContextData);
    }

    private Device device = null;
    private String deviceName = null;
    private String propertyName = null;
    private String lastDeviceName = null;
    private String lastPropertyName = null;

    private boolean active  = false;
    private boolean gotHigh = false;
    private boolean gotLow  = false;
    private boolean firstCall = true;

    private Color penColor = null;
    //private CAxis axis = new CAxis(CConstants.AXIS_LEFT);
    private double displayHi = 100;
    private double displayLo = 0;
    private double value = BAD_VALUE;

    /**
     * Null constructor. 
     */
    public CPen () {

    }

    /**
     * Constructor with parameters
     *
     * @param dName     the name of device this Pen connected
     * @param pName   the name of property this Pen connected
     */
    public CPen (String dName, String pName) {
	setDeviceAndPropertyName(dName, pName);
    }

 
    /**
     * Set limitsContext to get all meta-data at first time
     */
    synchronized protected void getFirstValue () 
    {
       try {
          if (deviceName == null || propertyName == null)   return;
          if (!deviceName.equals(lastDeviceName)) {
             device = new Device(deviceName);
             lastDeviceName = deviceName;
             }
          device.setContext (limitsContext);
          device.get(propertyName, this);
          firstCall = true; 
          }
       catch(Exception e) { e.printStackTrace(); }
     }


    /**
     * Set the Device name 
     * @param newName     the name of device you want to connect
     */
    synchronized public void setDeviceName(String newName) 
    {
       newName = CUtility.makeValidName(newName);
       if (!CUtility.testNameMatch(deviceName, newName)) {
          stopMonitor();
          deviceName = newName;
          getFirstValue ();
       } 
    }

    /**
     * Return the device name to which this component is connected.
     * If device name is not set, return empty string "". 
     */
    public String getDeviceName()
    {
       return (deviceName==null)?BLANK:deviceName;
    }

    /**
     * Set device property name 
     * @param newName    the name of device property you want to connect
     */  
    synchronized public void setPropertyName(String newName)
    {
        newName = CUtility.makeValidName(newName);
        if (!CUtility.testNameMatch(propertyName, newName)) {
           stopMonitor();
           propertyName = newName; 
           getFirstValue ();        
        }
    }

   /**
     * Return the device property name to which this component is connected.
     * If property name is not set, return empty string "". 
     */
    public String getPropertyName()
    {
        return (propertyName==null)?BLANK:propertyName;
    }

   /**
     * Set device name and device property name
     * @param newDeviceName    the device name you want to connect
     * @param newPropertyName  the device property name you want to connect
     */  
    synchronized public void setDeviceAndPropertyName
                                       (String newDeviceName,
					String newPropertyName)
    {
	newDeviceName = CUtility.makeValidName(newDeviceName);
	newPropertyName = CUtility.makeValidName(newPropertyName);
        if ((!CUtility.testNameMatch(deviceName, newDeviceName)) ||
           (!CUtility.testNameMatch(propertyName, newPropertyName))) {
           stopMonitor();
	   deviceName = newDeviceName;
           propertyName = newPropertyName; 
           getFirstValue ();        
        }
    }

    protected void stopMonitor() 
    {
        if (active) {
	    try {
		active = false;
		device.monitorOff(propertyName,this);
	    } 
	    catch (IOException e) {
		e.printStackTrace();
	    }
	}
    }
    
    protected void startMonitor() 
    {
	try {
	    if (active) stopMonitor();
	    if (deviceName == null || propertyName == null) return;
	    if (!deviceName.equals(lastDeviceName)) {
		device = new Device(deviceName);
		lastDeviceName = deviceName;
	    }    
	    device.setContext (defContext);
	    device.monitorOn(propertyName,this);
	    lastPropertyName = propertyName;
	    active = true;
	} 
	catch (IOException e) {
	    active = false;
	    e.printStackTrace();
	}
    }
    
    /**
     * Set the color of Pen
     */
    public void setPenColor(Color aColor){
	penColor = aColor;
    }
    
    /**
     * Return the color of Pen
     */
    public Color getPenColor () {
	return penColor;
    }
    
    /** 
     * Set display range. Use min as max if the min > max.
     *
     * @param min     the display low value
     * @param max     the display high value
     */
    public void setDisplayRange (double min, double max)
    {
	displayLo = Math.min(min, max);
	displayHi = Math.max(min, max);
	gotHigh = true;
	gotLow = true;
    }


    /**
     * set display high.
     *
     * @param hi     the display high value
     */
    public void setDisplayHi (double hi)
    {
	displayHi = hi;
	gotHigh = true;
    }

    /**
     * set display low.
     *
     * @param lo     the display low value
     */
    public void setDisplayLo (double lo)
    {
	displayLo = lo;
	gotLow = true;
    }


    /**
     * Return display high
     */
    public double getDisplayHi () { return displayHi; }
    
    /**
     * Return display low
     */
    public double getDisplayLo () { return displayLo; }

    public double getNewValue() {
	//System.out.println("Pen:: return value = "+value);
	return value;
    }

    protected boolean gotRange() { return !firstCall; }

    public void deviceChanged(DeviceEvent de) {
	if (firstCall) {
	    try {
		value = de.getData().getDataEntry("value").doubleValue();
		if (!gotHigh) {
		    displayHi=de.getData().getDataEntry("displayHigh").doubleValue();
		    gotHigh = true;
		}
		if (!gotLow) {
		    displayLo=de.getData().getDataEntry("displayLow").doubleValue();
		    gotLow = true;
		}
		firstCall = false;
		startMonitor();
	    } catch (Exception e) {
		//e.printStackTrace();
		System.out.print("failed to get display range");
	    }
	}
	else {
	    try {
		value = de.getData().getDataEntry("value").doubleValue();
	    } catch (Exception e) {
		//e.printStackTrace();
		System.out.print("failed to get value");
	    }
	}
    }
}
    












