/*
 * Copyright (c) 2000 Southeastern Universities Research Association,
 *                    Continuous Electron Beam Accelerator Facility
 *
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 *
 *  Revision History:
 *    03/10/00:  Release version 0.1.
 *    04/17/00:  Release version 0.2.  
 *
 */

package adm;

import java.awt.*;
import java.awt.event.*;

import cdev.*;
import cdev.data.*;

/**
 * CSlider is an awt component bean. It provide a convenient means for 
 * allowing a user to control a device property's value in a control system.
 * The size, bound, foreground, background and the font of the label on the 
 * slider can be set like an awt component. 
 * <BR><BR>
 * There are four label styles which can be selected, with a default of 
 * LABEL_NONE. If LABEL_AXIS is used, tick marks and display range will 
 * appear. The property's value will also shown if LABEL_VALUE is selected. 
 * All the information including the device and property name will on the 
 * widget when the LABEL_ALL is used. 
 * <BR><BR>
 * The orientation of the CSilder widget can be either VERTICAL (default)
 * or HORIZONTAL.
 * <BR><BR>
 * There are two ways to adjust the value: (1) dragging the thumb and 
 * (2) clicking the face of the slider. When the thumb is dragged, only a 
 * change larger than precision will be sent to the control system. The 
 * precision value will be obtained from control system if it is available.
 * Otherwise the default value of 1% of the control range will be used.
 * When clicking on the face, the value of the property will increase or 
 * decrease a unit increment depending upon the clicking position. 
 * By default the unit increment is the same as the precision, and they 
 * can be changed using either the setPrecision(double) or 
 * setUnitIncrement(double) method. Double clicking the thumb, a new window 
 * will pop up to allow the user to change the unit increment value on the 
 * fly. Create a slider component. It is used to control the device 
 * property value.
 *
 *
 * @version   0.2  04/17/00
 * @author    Ying Chen, Chip Watson
 *            (Based on work by Cuong Quach and Mr. John Winans)
 * @since     adm0.1
 *
 */
public class CSlider extends adm.CIndicator implements 
             MouseListener, MouseMotionListener {
    
    private Data tdata = new Data();
    private static int VAL_TAG = Data.convertKey("value");
    private int thumb1X, thumb1Y, thumb1W, thumb1H;
    private int thumb2X, thumb2Y, thumb2W, thumb2H;  
    private boolean eventLock = false;
    private double unitIncrement = 1;
    private SettingsBox	setIncrementFrame;


    /**
     * Null Constructor.
     */
    public CSlider() 
    {
	this(0, 0);	
    }

    /**
     * Constructor with device name and property name
     *
     * @param dName    device name
     * @param pName    property name
     */
    public CSlider(String dName, String pName)
    {
        super(dName, pName);
    }



    /**
     * Constructor.
     * Defines the canvas size and use default foreground and background color
     * @param width    component width
     * @param height   component height
     */
    public CSlider (int w, int h) 
    {
	super(w, h);	
	addMouseListener(this);
	addMouseMotionListener(this);
    }

    
    /**
     * Constructor
     * Defines the canvas size and sets foreground and background colors.
     * @param width     component width
     * @param height    component height
     * @param fg        foreground color
     * @param bg        background color
     */
   public CSlider (int w, int h, Color fg, Color bg) 
    {
	this(w, h);
	setForeground(fg);
	setBackground(bg);
    }
    
   
   protected void initGraphics () 
    {
	halfThumb = 10;
	rangeHi = controlHi;
	rangeLo = controlLo;
	super.initGraphics ();
	precision = (rangeHi - rangeLo)/100;
	unitIncrement = precision;
    }



    /**
     * Set control high and control low. The range style property
     * will be forced to RANGE_STATIC.
     * 
     * @param lo   the display low
     * @param Hi   the display high
     * 
     * hi must be larger than lo otherwise nothing is changed.
     */
    public void setControlRange (double lo, double hi)
    {
	if (hi > lo) {
	    controlHi = hi;
	    controlLo = lo;
	    rangeHi = controlHi;
	    rangeLo = controlLo;
	    rangeStyle = RANGE_STATIC;
	    offGraphics = null;
	}
    }

    /**
     * Set control high. The range style property will be 
     * forced to RANGE_STATIC.
     * 
     * @param Hi   the display high
     * 
     * hi must be larger than controlLo otherwise nothing is changed.
     */
    public void setControlHi (double hi)
    {
	if (hi > controlLo) {
	    controlHi = hi;
	    rangeHi = controlHi;
	    rangeStyle = RANGE_STATIC;
	    offGraphics = null;
	}
    }

    /**
     * Set control low. The range style property will be 
     * forced to RANGE_STATIC.
     * 
     * @param lo   the display low
     * 
     * lo must be smaller than controlHi otherwise nothing is changed.
     */
    public void setControlLo (double lo)
    {
	if (lo < controlHi) {
	    controlLo = lo;
	    rangeLo = controlLo;
	    rangeStyle = RANGE_STATIC;
	    offGraphics = null;
	}
    }

    /**
     * Return control high
     */
    public double getControlHi () { return controlHi; }


    /**
     * Return control low
     */
    public double getControlLo () { return controlLo; }


    /**
     * Set unit increment value. This is the value change when user clicks
     * the slider face. Default value is 1% of control range.
     */
    public void setUnitIncrement (double newUnitIncrement) 
    {
	unitIncrement = newUnitIncrement;
    }


    /**
     * Return the unit increment value.
     */
    public double getUnitIncrement ()
    {
	return unitIncrement;
    }

    /**
     * Set precision. This value is used to control the rate at which
     * the data is sent to control system when the user is dragging 
     * the thumb of slider. Default value is 1% of control range.
     */
    public void setPrecision (double newPrecision) 
    {
	precision = newPrecision;
    }

    /**
     * Return the precision value.
     */
    public double getPrecision () 
    {
	return precision;
    }


    protected void paintForeground (Graphics g) 
    {
	//-------------------------------------------------
	// draw value when LABEL_VALUE and LABEL_ALL
	//-------------------------------------------------   
	if ((labelStyle == LABEL_VALUE)||(labelStyle == LABEL_ALL)) {
	    drawValue(g);
	}

        //--------------------------------------------
        // Erase old fill rect.
        //--------------------------------------------
        g.setColor (bgColor);
        if (orientation  == VERTICAL)
	    g.fill3DRect(fillX, fillY, fillW, maxFillH, false);
	else
	    g.fill3DRect(fillX, fillY, maxFillW, fillH, false);

        //------------------------------------------------------
        // recalculate fill% (1.0< = fill% >= 0.0)
        //------------------------------------------------------ 
        fillPercent = (currVal - rangeLo)/(rangeHi - rangeLo);
        fillPercent = Math.min(fillPercent, 1.0);   
        fillPercent = Math.max(fillPercent, 0.0);       
	if (orientation == VERTICAL) {
	    fillH = (int)((maxFillH - 2*halfThumb)*fillPercent); 
	    thumb1X = fillX;
	    thumb1Y = fillY + maxFillH - halfThumb - fillH;
	    thumb1W = fillW;
	    thumb1H = halfThumb;

	    thumb2X = thumb1X;
	    thumb2Y = fillY + maxFillH - fillH - 2*halfThumb;
	    thumb2W = fillW;
	    thumb2H = halfThumb;
	}
	else {
	    fillW = (int)((maxFillW - 2*halfThumb)*fillPercent);
	    thumb1X = fillX + fillW;
	    thumb1Y = fillY;
	    thumb1W = halfThumb;
	    thumb1H = fillH;

	    thumb2X = fillX + fillW + halfThumb;
	    thumb2Y = thumb1Y;
	    thumb2W = halfThumb;
	    thumb2H = fillH;
	}

	//------------------------------------------------//
        // Draw new pointer
        //------------------------------------------------//
	if (colorMode == CLRMOD_ALARM) 
	    g.setColor(currColor);
        g.fill3DRect (thumb1X, thumb1Y, thumb1W, thumb1H, true);
        g.fill3DRect (thumb2X, thumb2Y, thumb2W, thumb2H, true);

    }
 

    public void deviceChanged(DeviceEvent event) {
        if (eventLock) return;
        super.deviceChanged(event);
    }


    /**
     *  MouseListener interface
     */
    public void mouseEntered  (MouseEvent me) {}


    /**
     *  MouseListener interface
     */
    public void mouseExited  (MouseEvent me) {}


    /**
     * Handles mouse click.
     * Click on the face, the value will change a unit increment.
     * If double click the thumb, pop up a new window to allow the
     * user to change the unit increment value.
     */
    public void mouseClicked (MouseEvent me)
    {
	int x = me.getX();
	int y = me.getY();
	//--- click the thumb -------------
	if(inThumb(x, y)) {
	    if (me.getClickCount() > 1) {         //double click
		if (setIncrementFrame != null) {
		    setIncrementFrame.show();
		    setIncrementFrame.toFront();
		}
		setIncrementFrame = new SettingsBox(this);
		setIncrementFrame.setBounds(200, 200, 250, 150);
		setIncrementFrame.show();
	    }
	}
	//--- click the face --------------
	if(inFace(x, y)) {
	    if (orientation == VERTICAL) {
		if (y < thumb2Y)           { currVal += unitIncrement; }
		if (y > thumb1Y+halfThumb) { currVal -= unitIncrement; }
		}
	    else {
		if (x > thumb2X+halfThumb) { currVal += unitIncrement; }
		if (x < thumb1X)           { currVal -= unitIncrement; }
		}
	    tdata.insert(VAL_TAG, currVal);
	    try {
		device.set (propertyName, tdata, (DeviceListener)null);
		device.getControlSystem().flush();
		//System.out.println("send value+increment ("+currVal+
		//                   ") to control system");
	    } catch (Exception ex) {
		System.out.println ("failed to send value\n"+ex);
	    }
	    repaint();
	}
    }


    /**
     * Press the mouse in the thumb area to initiate a drag, 
     * and monitoring will be suspended.
     */
    public void mousePressed (MouseEvent me)
    {
	if(inThumb(me.getX(), me.getY())) 
	   eventLock = true;
    }


    /**
     * Handle the mouse release.
     * Release the mouse to terminate a drag and resume monitoring.
     */
    public void mouseReleased (MouseEvent me)
    {
	eventLock = false;
    }


    /**
     * Handle the mouse drag.
     * Only a change larger than precision will be sent.
     */
    public void mouseDragged (MouseEvent me) 
    {
	int x = me.getX();
	int y = me.getY();
	if (eventLock) {
	    //System.out.println ("mouse drag to ("+x+","+y+")");
	    if (orientation == VERTICAL) 
		fillPercent=(fillY+maxFillH-halfThumb-y)/(double)(maxFillH-2*halfThumb);
	    else
		fillPercent=(x-fillX-halfThumb)/(double)(maxFillW-2*halfThumb);
	    fillPercent = Math.min(fillPercent, 1.0);   
	    fillPercent = Math.max(fillPercent, 0.0);       
	    double value = fillPercent*(rangeHi - rangeLo) + rangeLo;
	    if (Math.abs(currVal - value) > precision) {
		tdata.insert(VAL_TAG, value);
		try {
		    device.set (propertyName, tdata, (DeviceListener)null);
		    device.getControlSystem().flush();
		    //System.out.println("send new value("+value+") to control system");
		} catch (Exception ex) {
		    System.out.println ("failed to send value\n"+ex);
		}
	    }
	   currVal = value;     
	   repaint();
	}
    }

    /**
     * MouseMotionListener interface
     */
    public void mouseMoved (MouseEvent me) {}


    /**
     * This determines if the mouse is sitting in the slider rectangle.
     */
    private boolean inFace (int mx, int my)
    {
	if ( (mx > fillX) && (mx < (fillX + maxFillW)) &&
	     (my > fillY) && (my < (fillY + maxFillH)) ) {
	    return true;
	} else {
	    return false;
	}
    }


    /**
     * Determines if the mouse is sitting on the thumb.
     */
    private boolean inThumb (int mx, int my)
    {
	if (orientation == VERTICAL) {
	    if ( (mx > thumb2X) && (mx < (thumb2X + thumb2W)) &&
		 (my > thumb2Y) && (my < (thumb2Y + thumb2H + thumb1H)) ) 
		return true;
	}
	else {
	    if ( (mx > thumb1X) && (mx < (thumb1X + thumb1W + thumb2W)) &&
		 (my > thumb1Y) && (my < (thumb1Y + thumb1H)) ) 
		return true;
	}
	return false;
    }

} //end of CSlider


//**************************************************************************

/**
 * SettingsBox class creates a separate window for setting unit increment
 * parameters for CSlider widget.
 */
    class SettingsBox extends Frame implements ActionListener
{
    static final int FONT_SIZE = 12;
    private Button doneB, increaseB, decreaseB, cancelB;
    private TextField inputTF;
    private Label directions;
    protected CSlider widget;
  
    private double currIncrement, maxHi, minLo;

    public SettingsBox(CSlider w) 
    { 
	super();
	widget = w;	   
	maxHi = (widget.getControlHi() - widget.getControlLo())/10;
	minLo = widget.getPrecision()/10;
	currIncrement = widget.getUnitIncrement();
	Font font1 = new Font("Dialog", Font.PLAIN, FONT_SIZE);
	Font font2 = new Font("Dialog", Font.BOLD, FONT_SIZE);

	directions = new Label("Please select a unit increment.");
	directions.setFont(font1);
	Panel northP = new Panel();
	northP.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 10));
	northP.add(directions);

	Panel centerP = new Panel();
	decreaseB = new Button("Decrease");
	decreaseB.setFont(font1);
	decreaseB.addActionListener(this);
	centerP.add(decreaseB);

	inputTF = new TextField(7);
	inputTF.setText(CUtility.format(currIncrement));
	inputTF.setFont(font2);
	inputTF.addActionListener(this);
	centerP.add(inputTF);

	increaseB = new Button("Increase");
	increaseB.setFont(font1);
	increaseB.addActionListener(this);
	centerP.add(increaseB);
	
	Panel southP = new Panel();
	southP.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
	doneB = new Button("Change");
	doneB.setFont(font1);
	doneB.addActionListener(this);
	southP.add(doneB);
	cancelB = new Button("Cancel");
	cancelB.setFont(font1);
	cancelB.addActionListener(this);
	southP.add(cancelB);

	setLayout(new BorderLayout());
	add(northP, BorderLayout.NORTH);
	add(centerP, BorderLayout.CENTER);
	add(southP, BorderLayout.SOUTH);
  }


    public void actionPerformed (ActionEvent e)
    {
	if (e.getSource() == cancelB) {
	    this.dispose();
	}
	else if ((e.getSource() == doneB) || (e.getSource() == inputTF)) {
	    try {
		currIncrement = Double.valueOf(inputTF.getText()).doubleValue();
	    }
	    catch (NumberFormatException ne) {
		directions.setText("Increment must be a number.");
		return;
	    }
	    if ((currIncrement > maxHi) || (currIncrement < minLo)) {
		directions.setText("Increment must in ("+minLo+", "+maxHi+")");
		if(currIncrement < maxHi)   increaseB.setEnabled(true);
		if(currIncrement > minLo)   decreaseB.setEnabled(true);  
		return;
	    }
	    widget.setUnitIncrement(currIncrement);
	    this.dispose();
	}

	else if (e.getSource() == decreaseB) {
	    currIncrement = Double.valueOf(inputTF.getText()).doubleValue();
	    currIncrement /= 2; 
	    if( currIncrement < minLo) {
		currIncrement = minLo;
		inputTF.setText(String.valueOf(currIncrement));
		decreaseB.setEnabled(false);
	    }
	    inputTF.setText(CUtility.format(currIncrement));
	    if(currIncrement < maxHi)   increaseB.setEnabled(true);
	    if(currIncrement > minLo)     decreaseB.setEnabled(true);  
    }
	else if (e.getSource() == increaseB) {
	    currIncrement = Double.valueOf(inputTF.getText()).doubleValue();
	    currIncrement *= 2;
	    if(currIncrement > maxHi) {
		currIncrement = maxHi;
		increaseB.setEnabled(false);
	    }
	    inputTF.setText(CUtility.format(currIncrement));
	    if(currIncrement < maxHi)   increaseB.setEnabled(true);
	    if(currIncrement > minLo)   decreaseB.setEnabled(true);
    }
	}
    
}  // end of SettingsBox









