/*
 * Copyright (c) 2000 Southeastern Universities Research Association,
 *                    Continuous Electron Beam Accelerator Facility
 *
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 *
 * Revision History:
 *   03/10/00:  Release version 0.1.
 *   04/17/00:  Release version 0.2 (add format property & key event)
 *   08/16/00:  Release version 0.4 (modify to work with string value)
 *
 */

package adm;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import cdev.*;
import cdev.data.*;

/**
 * CTextField is an awt component bean. It is used to control and monitor 
 * a value of the specified device property in the control system. Its size, 
 * bound, foreground, background and the font can be set by the user. 
 * <BR><BR>
 * The color of the text will be the same as the foreground if the color 
 * mode is set to CLRMOD_STATIC (default). When the color mode is set to 
 * CLRMOD_ALARM the color of the text will change to yellow or red if the 
 * property is in a warning or alarm state (determined by warning and alarm 
 * limits). 
 * <BR><BR>
 * The display value can be formatted following C "printf" conventions, or 
 * use one of following static format constants: 
 * <OL>
 *    <LI>FRMT_INTEGER (%d) 
 *    <LI>FRMT_FLOAT (%.3f) 
 *    <LI>FRMT_SCIENTIFIC (%.3e) 
 *    <LI>FRMT_AUTO (%.3g) 
 *    <LI>FRMT_HEX (%#x) 
 *    <LI>FRMT_OCTAL (%#o) 
 * </OL>
 * The default format is FRMT_AUTO which means the value will display as a 
 * float number or scientific notation depending on the value. 
 * <BR><BR>
 * When the cursor is focused on the widget, the monitor (live update) will 
 * stop. If no action from user occurs within 5 seconds, the live update 
 * will start again. This 5 second counter will restart after each key 
 * stroke delivered to the widget. 
 *
 *
 * @version   0.2  03/29/00
 * @author    Chip Watson, Ying Chen 
 *            (Based on work by Cuong Quach and Mr. John Winans)
 * @since     adm0.1
 *
 */
public class CTextField extends TextField implements CInterface, CConstants,
    DeviceListener, FocusListener, KeyListener, ActionListener, Runnable {

    private static int VAL_TAG = Data.convertKey("value");
    private static String BLANK = "";
    private static double BAD_VALUE = -11111;
    private static DeviceContext defContext = new DeviceContext();
    private static DeviceContext limitsContext;
    static {
      Data limitsContextData = new Data();
      limitsContextData.insert("displayHigh", 1);
      limitsContextData.insert("displayLow", 1);
      limitsContextData.insert("alarmHigh", 1);
      limitsContextData.insert("alarmLow", 1);
      limitsContextData.insert("warningHigh", 1);
      limitsContextData.insert("warningLow", 1);
      limitsContext = new DeviceContext(limitsContextData);
    }
    static final Color DEFAULT_WARNING_COLOR = Color.yellow;
    static final Color DEFAULT_ALARM_COLOR = Color.red;
    static final Color DEFAULT_FOREGROUND = Color.black;
    static final int DEFAULT_TIMEOUT_PERIOD = 5000;     // 5 sec. timeout
    static final int DEFAULT_SLEEP_TIME = 1000;     // 1 sec. check time

    private boolean active = false;
    private boolean eventLock = false;   
    private boolean firstCall = true; 
    private boolean gotLimits = false;

    //--- define default value --------------------------
    private Color warningColor = DEFAULT_WARNING_COLOR;
    private Color alarmColor   = DEFAULT_ALARM_COLOR;
    private Color fgColor = DEFAULT_FOREGROUND;
    private int colorMode  = CLRMOD_STATIC;
    private String format = FRMT_AUTO;
    private double displayHi = BAD_VALUE;
    private double displayLo = BAD_VALUE;
    private double alarmHi = BAD_VALUE;
    private double alarmLo = BAD_VALUE;
    private double warningHi = BAD_VALUE;
    private double warningLo = BAD_VALUE;
    private double value = BAD_VALUE;
    private String valueString = null;
    private Thread timer = null;
    private int counter = 0;

    private Cursor defCur  = new Cursor(Cursor.DEFAULT_CURSOR);
    private Cursor waitCur = new Cursor(Cursor.WAIT_CURSOR);
 
    private Data tdata = new Data();
    private Device device = null;
    private String deviceName = null;
    private String propertyName = null;
    private String lastDeviceName = null;
    private String lastPropertyName = null;
    private DeviceListener setListener = null;

    /**
     * Null Constructor
     */
    public CTextField()
    {
	setColumns(8);
	setForeground(fgColor);
	addFocusListener(this);
	addKeyListener(this);
	addActionListener(this);
    }
   
    /**
     * Null constructor  with device name and property name
     *
     * @param dName    device name
     * @param pName    property name
     */
    public CTextField(String dName, String pName)  {
        this();
        setDeviceName(dName);
        setPropertyName(pName);
    }


   /**
     * Set the color mode property
     * @param newColorMode   the new color mode, which may be one of:
     *             <OL>
     *             <LI>  CConstants.CLRMOD_STATIC
     *             <LI>  CConstants.CLRMOD_ALARM
     *             </OL>
     */
    public void setColorMode (int newColorMode) {
	colorMode = newColorMode;
    }
    
    
    /*
     * Return the color mode property
     */
    public int getColorMode () { return colorMode; }


    /**
     * Return the text format (format is following the printf conventions)
     */
    public String getFormat() { return format; }

    /**
     * Set the text format. The label will be forced to display the value of 
     * the specified device property.
     * @param format    the format of the label text, which following printf 
     *                  conventions. Or use one of following string:
     *                  <OL>
     *                  <LI> CConstants.FRMT_INTEGER
     *                  <LI> CConstants.FRMT_FLOAT   
     *                  <LI> CConstants.FRMT_SCIENTIFIC
     *                  <LI> CConstants.FRMT_AUTO
     *                  <LI> CConstants.FRMT_HEX   
     *                  <LI> CConstants.FRMT_OCTAL
     *                  </OL>
     */
    public void setFormat(String aFormat) { format = aFormat; }

   
    /**
     * Start a new thread (for focus timeout)
     */
    public void start()
    {
	if (timer == null) { 
	    timer = new Thread(this);
	    timer.start();
	}
    }

    /**
     * Stop the focus timeout thread 
     */
    public void stop() 
    {
	if (timer != null && timer.isAlive()) 
	    timer.stop();
	timer = null;
    }
   
    /**
     * Sleep a timeout period then remove the focus
     */
    public void run()
    {
	try {
	    while (counter <= DEFAULT_TIMEOUT_PERIOD) {
		Thread.sleep(DEFAULT_SLEEP_TIME);
		counter += DEFAULT_SLEEP_TIME;
	    }
	} catch (InterruptedException ie) {
	    System.out.println("thread sleep failed"); 
	}
	eventLock = false;
	getParent().requestFocus();   //move focus to parent
    }

    /**
     * for KeyListener interface
     * Reset the timeout counter
     */
    public void keyTyped (KeyEvent e) {
	if (e.getKeyChar() != KeyEvent.VK_ENTER) {
	    //System.out.println("restart the timer");
	    counter = 0;
	}
    }

    /**
     * for KeyListener interface
     */
    public void keyPressed (KeyEvent e) {}
    public void keyReleased (KeyEvent e) {}

    /**
     * for FocusListener interface
     * Stop the monitor when component gains focus, so that the user can
     * enter the new update value. At same time start a new timeout thread.
     * If no value is entered (<return>) in 5 seconds, the focus is discarded
     * and monitoring is resumed.
     */ 
    public void focusGained (FocusEvent evt)
    {
	eventLock = true;
	//System.out.println("gained focus and started timer");
	counter = 0;
	start();
    }


    /**
     * for FocusListener interface
     * Start the monitor and stop the timer thread.
     */
    public void focusLost (FocusEvent evt) { 
	eventLock = false;
	stop();
    }

    /**
     * Send the new value to the control system and resume monitoring
     */
    public void actionPerformed(ActionEvent evt) 
    {
	if (setListener==null) setListener = new DeviceListener() {
            public void deviceChanged(DeviceEvent evt) {
                if (evt.getError() == null) 
		    System.out.println("success to send value");
		else {
		    System.out.println("failed to send value");
		    reSetText();
		}
		stop();
		transferFocus();  
		eventLock = false;
		setCursor(defCur);
	    }
        };

	if (eventLock) {
	    tdata.insert(VAL_TAG, getText());
	    try {
		device.set(propertyName, tdata, setListener);
		device.getControlSystem().flush();
		setCursor(waitCur);
	    } catch (Exception e) {
		System.out.println("failed to set value: "+e);
	    }
	    //transferFocus();  
	    //eventLock = false;
	}
    }

    /**
     * Set limitsContext to get all meta-data at first time
     */
    synchronized protected void getFirstValue () 
    {
       try {
          if (deviceName == null || propertyName == null)   return;
          if (!deviceName.equals(lastDeviceName)) {
             device = new Device(deviceName);
             lastDeviceName = deviceName;
             }
          device.setContext (limitsContext);
          device.get(propertyName, this);
          firstCall = true; 
          }
       catch(Exception e) { e.printStackTrace(); }
     }
     

    /**
     * Set the Device name 
     * @param newName     the name of the device to monitor
     */
    synchronized public void setDeviceName(String newName)
    {
	newName = CUtility.makeValidName(newName);
	if (!CUtility.testNameMatch(deviceName, newName)) {
	    stopMonitor();
	    deviceName = newName;
	    getFirstValue ();
	}
    }


    /**
     * Return the device name to which this component is connected.
     * If device name is not set, return empty string "".
     */
    public String getDeviceName()
    {
	return (deviceName==null)?BLANK:deviceName;
    }

    /**
     * Set device property name 
     * @param newName    the name of device property to monitor
     */  
    synchronized public void setPropertyName(String newName)
    {
	newName = CUtility.makeValidName(newName);
	if (!CUtility.testNameMatch(propertyName, newName)) {
	    stopMonitor();
	    propertyName = newName;
	    getFirstValue ();
	}
    }
    
    /**
     * Return the device property name to which this component is connected.
     * If property name is not set, return empty string "". 
     */
    public String getPropertyName()
    {
	return (propertyName==null)?BLANK:propertyName;
    }
    

    protected void stopMonitor() {
	if (active) try {
	    active = false;
	    device.monitorOff(propertyName,this);
	} catch (IOException e) {
	    e.printStackTrace();
	}
    }
    
    protected void startMonitor() {
	try {
	    if (active) stopMonitor();
	    if (deviceName == null || propertyName == null) return;
	    if (!deviceName.equals(lastDeviceName)) {
		device = new Device(deviceName);
		lastDeviceName = deviceName;
	    } 
	    device.setContext (defContext);
	    device.monitorOn(propertyName,this);
	    lastPropertyName = propertyName;
	    active = true;
	} catch (IOException e) {
	    active = false;
	    e.printStackTrace();
	}
    }
	

    public void deviceChanged(DeviceEvent evt) {
	if (firstCall) {
	    fgColor = getForeground();
	    try {
		displayHi=evt.getData().getDataEntry("displayHigh").doubleValue();
		displayLo=evt.getData().getDataEntry("displayLow").doubleValue();
		gotLimits = true;
	    }
	    catch (Exception e) {
		gotLimits = false;
		System.out.println("Failed to get display range");
	    }
	    
	    //--- get the alarm and warning range ----------------------
	    try {
		alarmHi=evt.getData().getDataEntry("alarmHigh").doubleValue();
		alarmLo=evt.getData().getDataEntry("alarmLow").doubleValue();
	    }
	    catch  (Exception e) {
		gotLimits = false;
		System.out.println("Failed to get alarm range");
	    }
	    try {
		warningHi=evt.getData().getDataEntry("warningHigh").doubleValue();
		warningLo=evt.getData().getDataEntry("warningLow").doubleValue();
	    }
	    catch  (Exception e) {
		gotLimits = false;
		System.out.println("Failed to get warining range");
	    }
	    firstCall = false;	
	    startMonitor();
	}
	else {
	    if (eventLock) return;
	    try { 
		valueString = evt.getData().getDataEntry("value").stringValue();
		reSetText();
	    } catch (Exception e) { 
		setText(BLANK);	
		System.out.println("CTextField:: failed to get the value");
	    }
	}
    }
    

    private void reSetText () {
	try {
	    value = Double.valueOf(valueString).doubleValue();
	    if ((colorMode == CLRMOD_ALARM) && gotLimits) {
		if ((value-alarmHi > 0) || (value-alarmLo<0)) 
		    setForeground(alarmColor);
		else if ((value-warningHi > 0) || (value-warningLo<0))
		    setForeground(warningColor);
		else
		    setForeground(fgColor);
	    }
	    setText(CUtility.format(value, format));
	} catch (Exception e) {
	    setText(valueString);
	}
    }


    /*   
    static public void main(String args[])
    {
	class DriverFrame extends Frame {
	    public DriverFrame() {
		addWindowListener(new WindowAdapter() {
		    public void windowClosing(WindowEvent event)
			{
			    dispose();	  // free the system resources
			    System.exit(0); // close the application
			}
		});
		this.setLayout(null);
		this.setSize(400,300);
		this.add(new CTextField());
	    }
	}
	
	new DriverFrame().show();
    }
    */
}
