/*
 * Copyright (c) 2000 Southeastern Universities Research Association,
 *                    Continuous Electron Beam Accelerator Facility
 *
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 *
 * Revision History:
 *   03/10/00:  Release version 0.1.
 *   04/17/00:  Release version 0.2. (add set the data)
 *   08/07/00:  Release version 0.4. (move to adm.event package)
 *
 */

package adm.event;

import java.io.*;
import java.util.*;
import cdev.*;
import cdev.data.*;
import adm.*;

/**
 * CAdaptor is an invisible bean used to monitor and to control a cdev 
 * Device Property's value. 
 * <BR>
 * A cdev.DeviceListener can subscribe to this bean to receive notification 
 * of monitors. Objects can also send CDEV "set" commands via this bean. 
 * Currently the data type to set is restricted to int, double or String. 
 *
 *
 * @version   0.2  04/17/00
 * @author    Chip Watson, Ying Chen
 *            (Based on work by Cuong Quach and Mr. John Winans)
 * @since     adm0.1
 */
public class CAdaptor implements CInterface, DeviceListener
{
    static final int VAL_TAG = Data.convertKey("value");
    static final String BLANK = "";
    private boolean active = false;
     
    private ControlSystem sys;
    private Device device = null;
    private String deviceName = null;
    private String propertyName = null;
    private String lastDeviceName = null;
    private String lastPropertyName = null;
    
    private Vector listeners;
    private DeviceEvent event;

    public CAdaptor()
    {
	listeners = new Vector();
	try {
	    sys = ControlSystem.getDefault();
	} catch (IOException e) {
	    System.out.println("CAdaptor FAILURE");
	    e.printStackTrace();
	}
    }
    

    /**
     * Set the Device name 
     * @param newName     the name of device you want to connect
     */   
    synchronized public void setDeviceName(String newName)
    {
        newName = CUtility.makeValidName(newName);
        if (!CUtility.testNameMatch(deviceName, newName)) {
            stopMonitor();
            deviceName = newName;
            startMonitor();
        }
    }
    

     /**
     * Return the device name to which this component is connected.
     * If device name is not set, return empty string "". 
     */
    public String getDeviceName() {
        return (deviceName==null)?BLANK:deviceName;
    }
    

   /**
     * Set device property name 
     * @param newName    the name of device property you want to connect
     */  
    synchronized public void setPropertyName(String newName)
    {
        newName = CUtility.makeValidName(newName);
        if (!CUtility.testNameMatch(propertyName, newName)) {
            stopMonitor();
            propertyName = newName;
            startMonitor();
        }
    }


    /**
     * Return the device property name to which this component is connected.
     * If property name is not set, return empty string "". 
     */
    public String getPropertyName() 
    {
	return (propertyName==null)?BLANK:propertyName;
    }
    

    /**
     * Add the device listener
     */
    synchronized public void addDeviceListener(DeviceListener dl) {
        listeners.addElement(dl);
    }

 
    /**
     * Remove the device listener
     */
    synchronized public void removeDeviceListener(DeviceListener dl) {
        listeners.removeElement(dl);
    }
    

    /**
     * Return device name.property name as a string
     */
    public String toString()
    {
        return deviceName+"."+propertyName;
    }
    

    protected void stopMonitor() {
        if (active) try {
            active = false;
            device.monitorOff(propertyName,this);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    

    protected void startMonitor() {
        try {
            if (active) stopMonitor();
            if (deviceName == null || propertyName == null) return;
            if (!deviceName.equals(lastDeviceName)) {
                device = new Device(deviceName);
                lastDeviceName = deviceName;
            }
            System.out.println("CLabel monitorOn "+deviceName
                               +" "+propertyName);
            device.monitorOn(propertyName,this);
            lastPropertyName = propertyName;
            active = true;
        } catch (IOException e) {
            active = false;
            e.printStackTrace();
        }
    }


    public void deviceChanged(DeviceEvent evt) {
        event = evt;
        Vector copy;
        event.setSource(this);
        synchronized (this) { copy = (Vector) listeners.clone(); }
        for (int i=0; i<copy.size();i++) {
            ((DeviceListener)copy.elementAt(i)).deviceChanged(event);
        }
    }
    

    /**
     * Return the data value as a string
     */
    public String getStringValue() 
    {
        return (event==null)?null:event.getData().getDataEntry("value").stringValue();
    }


    /**
     * Send a String data to control system
     */
    public void setValue( String val) 
    {            
	Data tdata = new Data();
	tdata.insert(VAL_TAG, val);
	try {
	    device.set (propertyName, tdata, (DeviceListener) null);
	    device.getControlSystem().flush();
	} catch (IOException ex) {    
	    System.out.println ("failed to send value\n"+ex);
	}
    }

    /**
     * Send a double data to control system
     */
    public void setValue (double val) 
    {
	Data tdata = new Data();
	tdata.insert(VAL_TAG, val);
	try {
	    device.set (propertyName, tdata, (DeviceListener) null);
	    device.getControlSystem().flush();
	} catch (IOException ex) {    
	    System.out.println ("failed to send value\n"+ex);
	}
    }

    /**
     * Send a int data to control system
     */
    public void setValue (int val)
    {
	Data tdata = new Data();
	tdata.insert(VAL_TAG, val);
	try {
	    device.set (propertyName, tdata, (DeviceListener) null);
	    device.getControlSystem().flush();
	} catch (IOException ex) {    
	    System.out.println ("failed to send value\n"+ex);
	}
    }
}





